----------------------------------------------------
--  COMS W4115 PLT - 2014 Spring Term Project
--              HL-HDL compiler
--             Woon Lee (wgl2107)
--        < Built-in component library >
----------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

package lib_comp is

-- Components declaration
	component LATCH is
	port (
		RST : in  std_logic;
		EN	: in  std_logic;
		CLK : in  std_logic;
		LAT_IN : in  std_logic;
		LAT_OUT : out std_logic
	);
	end component LATCH;
	
	component LATCH_n_bit is
	generic(num_bits : positive := 2);
	port (
		RST : in  std_logic;
		EN	: in  std_logic;
		CLK : in  std_logic;
		LAT_IN : in  std_logic_vector((num_bits-1) downto 0);
		LAT_OUT : out std_logic_vector((num_bits-1) downto 0)
	);
	end component LATCH_n_bit;

	component MUX is
	port (
		MUX_OUT : out  std_logic;
		EN	: in  std_logic;
		OUT_SEL : in  std_logic_vector(1 downto 0);
		MUX_IN0 : in  std_logic;
		MUX_IN1 : in  std_logic;
		MUX_IN2 : in  std_logic;
		MUX_IN3 : in  std_logic
	);
	end component MUX;
	
	component MUX_n_bit is
	generic(num_bits : positive := 2);
	port (
		MUX_OUT : out  std_logic_vector((num_bits-1) downto 0);
		EN	: in  std_logic;
		OUT_SEL : in  std_logic_vector(1 downto 0);
		MUX_IN0 : in  std_logic_vector((num_bits-1) downto 0);
		MUX_IN1 : in  std_logic_vector((num_bits-1) downto 0);
		MUX_IN2 : in  std_logic_vector((num_bits-1) downto 0);
		MUX_IN3 : in  std_logic_vector((num_bits-1) downto 0)
	);
	end component MUX_n_bit;

end lib_comp;


-- Entities and architectures of the components
library ieee;
use ieee.std_logic_1164.all;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

use work.globals.all;
use work.lib_comp.all;

ENTITY LATCH is
	port (
		RST : in  std_logic;
		EN	: in  std_logic;
		CLK : in  std_logic;
		LAT_IN : in  std_logic;
		LAT_OUT : out std_logic
	);
	end LATCH;

architecture RTL of LATCH is
begin
	proc_latch: process(RST, EN, CLK)
    begin
        if RST = '1' then
            LAT_OUT <= 'Z';
        elsif rising_edge(CLK) and EN = '1' then
            LAT_OUT <= LAT_IN;
        end if;
    end process proc_latch;
end RTL;

configuration CFG_LATCH of LATCH is
	for RTL
	end for;
end CFG_LATCH;

-------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

use work.globals.all;
use work.lib_comp.all;

ENTITY LATCH_n_bit is
	generic(num_bits : positive := 2);
	port (
		RST : in  std_logic;
		EN	: in  std_logic;
		CLK : in  std_logic;
		LAT_IN : in  std_logic_vector((num_bits-1) downto 0);
		LAT_OUT : out std_logic_vector((num_bits-1) downto 0)
	);
	end LATCH_n_bit;

architecture RTL of LATCH_n_bit is
begin
	proc_latch: process(RST, EN, CLK)
    begin
        if RST = '1' then
            LAT_OUT <= (others => 'Z');
        elsif rising_edge(CLK) and EN = '1' then
            LAT_OUT <= LAT_IN;
        end if;
    end process proc_latch;
end RTL;

configuration CFG_LATCH_n_bit of LATCH_n_bit is
	for RTL
	end for;
end CFG_LATCH_n_bit;

-------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

use work.globals.all;
use work.lib_comp.all;

ENTITY MUX is
	port (
		MUX_OUT : out  std_logic;
		EN	: in  std_logic;
		OUT_SEL : in  std_logic_vector(1 downto 0);
		MUX_IN0 : in  std_logic;
		MUX_IN1 : in  std_logic;
		MUX_IN2 : in  std_logic;
		MUX_IN3 : in  std_logic
	);
	end MUX;

architecture RTL of MUX is
begin
	proc_MUX : process (EN, OUT_SEL)
	begin
		if EN = '0' then MUX_OUT <= 'Z';
		else
			case OUT_SEL is
				when "00" =>	MUX_OUT <= MUX_IN0;
				when "01" =>	MUX_OUT <= MUX_IN1;
				when "10" =>	MUX_OUT <= MUX_IN2;
				when "11" =>	MUX_OUT <= MUX_IN3;
				when others =>	MUX_OUT <= 'Z';
			end case;
		end if;
	end process;
end RTL;

configuration CFG_MUX of MUX is
	for RTL
	end for;
end CFG_MUX;

-------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

use work.globals.all;
use work.lib_comp.all;

ENTITY MUX_n_bit is
	generic(num_bits : positive := 2);
	port (
		MUX_OUT : out  std_logic_vector((num_bits-1) downto 0);
		EN	: in  std_logic;
		OUT_SEL : in  std_logic_vector(1 downto 0);
		MUX_IN0 : in  std_logic_vector((num_bits-1) downto 0);
		MUX_IN1 : in  std_logic_vector((num_bits-1) downto 0);
		MUX_IN2 : in  std_logic_vector((num_bits-1) downto 0);
		MUX_IN3 : in  std_logic_vector((num_bits-1) downto 0)
	);
	end MUX_n_bit;

architecture RTL of MUX_n_bit is
begin
	proc_MUX : process (EN, OUT_SEL)
	begin
		if EN = '0' then MUX_OUT <= (others => 'Z');
		else
			case OUT_SEL is
				when "00" =>	MUX_OUT <= MUX_IN0;
				when "01" =>	MUX_OUT <= MUX_IN1;
				when "10" =>	MUX_OUT <= MUX_IN2;
				when "11" =>	MUX_OUT <= MUX_IN3;
				when others =>	MUX_OUT <= (others => 'Z');
			end case;
		end if;
	end process;
end RTL;

configuration CFG_MUX_n_bit of MUX_n_bit is
	for RTL
	end for;
end CFG_MUX_n_bit;


